/**
 * Copyright 2012-15 Simon Andrews
 *
 *    This file is part of SeqMonk.
 *
 *    SeqMonk is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    SeqMonk is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with SeqMonk; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package uk.ac.babraham.SeqMonk.Displays.DistanceValuePlot;

import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;

import uk.ac.babraham.SeqMonk.SeqMonkApplication;
import uk.ac.babraham.SeqMonk.DataTypes.DataCollection;
import uk.ac.babraham.SeqMonk.DataTypes.DataStore;
import uk.ac.babraham.SeqMonk.DataTypes.Probes.ProbeList;

public class DistanceValueDialog extends JDialog implements ActionListener {

	/**
	 * This dialog sets up the distance value plot which allows you to 
	 * look at the relationship between the distance from a probe and
	 * the quantitated value it has
	 */
	
	private JComboBox fullListBox;
	private JComboBox targetListBox;
	private JSlider maxDistSlider;
	private JCheckBox allowNegativeDistancesBox;
	private DataStore data;
	
	private JPanel plotPanel = new JPanel();;
	
	
	public DistanceValueDialog (DataCollection collection) {
		super(SeqMonkApplication.getInstance(),"Distance Value Histogram ["+collection.getActiveDataStore().name()+"]");
		
		data = collection.getActiveDataStore();
		
		ProbeList [] allLists = collection.probeSet().getAllProbeLists();
		
		getContentPane().setLayout(new BorderLayout());
		
		JPanel optionsPanel = new JPanel();
		
		optionsPanel.setLayout(new GridBagLayout());
		
		GridBagConstraints gbc = new GridBagConstraints();
		
		gbc.gridx=1;
		gbc.gridy=1;
		gbc.weightx = 0.1;
		gbc.weighty = 0.1;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.insets = new Insets(5, 5, 5, 5);
		
		optionsPanel.add(new JLabel("Full List"),gbc);
		gbc.gridy++;
		optionsPanel.add(new JLabel("Max Dist"),gbc);
		gbc.gridy=1;
		gbc.gridx++;
		gbc.weightx=0.9;
		fullListBox = new JComboBox(allLists);
		fullListBox.setPrototypeDisplayValue("No longer than this please");
		fullListBox.addActionListener(this);
		optionsPanel.add(fullListBox,gbc);
		gbc.gridx++;
		gbc.weightx=0.1;
		optionsPanel.add(new JLabel("Target List"),gbc);
		gbc.gridx++;
		gbc.gridwidth = 2;
		gbc.weightx=0.9;
		targetListBox = new JComboBox(allLists);
		targetListBox.setPrototypeDisplayValue("No longer than this please");
		targetListBox.addActionListener(this);
		optionsPanel.add(targetListBox,gbc);
		
		gbc.gridy++;
		gbc.gridx=2;
		gbc.gridwidth = 4;
		
		JPanel maxDistPanel = new JPanel();
		maxDistPanel.setLayout(new BorderLayout());
		
		maxDistSlider = new JSlider(1,collection.genome().getLongestChromosomeLength(),100000);
		maxDistSlider.setPaintTicks(true);
		
		maxDistPanel.add(maxDistSlider,BorderLayout.CENTER);
		
		allowNegativeDistancesBox = new JCheckBox();
		allowNegativeDistancesBox.addActionListener(this);
		maxDistPanel.add(allowNegativeDistancesBox,BorderLayout.EAST);
		optionsPanel.add(maxDistPanel,gbc);
		
		getContentPane().add(optionsPanel,BorderLayout.NORTH);
		
		getContentPane().add(plotPanel,BorderLayout.CENTER);
		
		setSize(800,600);
		setLocationRelativeTo(SeqMonkApplication.getInstance());
		setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
		setVisible(true);
		
		
	}


	public void actionPerformed(ActionEvent ae) {
		// Get the new probe lists and make a new plot.
		
		ProbeList fullList = (ProbeList)fullListBox.getSelectedItem();
		ProbeList targetList = (ProbeList)targetListBox.getSelectedItem();
		
		DistanceValuePanel dvp = new DistanceValuePanel(data, fullList, targetList, maxDistSlider.getValue(),allowNegativeDistancesBox.isSelected());
		
		getContentPane().remove(plotPanel);
		plotPanel = dvp;
		
		getContentPane().add(plotPanel);
		
		validate();
		repaint();
		
		
	}
	
	
}
