/**
 * Copyright 2013-15 Simon Andrews
 *
 *    This file is part of SeqMonk.
 *
 *    SeqMonk is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    SeqMonk is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with SeqMonk; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package uk.ac.babraham.SeqMonk.Displays.VennDiagram;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;

import uk.ac.babraham.SeqMonk.SeqMonkApplication;
import uk.ac.babraham.SeqMonk.DataTypes.DataCollection;
import uk.ac.babraham.SeqMonk.DataTypes.Probes.ProbeList;
import uk.ac.babraham.SeqMonk.Utilities.ImageSaver.ImageSaver;

public class VennDiagramDialog extends JDialog {

	private JComboBox leftProbeListBox;
	private JComboBox rightProbeListBox;
	private VennDiagramPanel vennDiagramPanel = new VennDiagramPanel();
	
	public VennDiagramDialog (DataCollection collection) {
		super(SeqMonkApplication.getInstance(),"Venn Diagram");
		
		setLayout(new BorderLayout());
		
		ProbeList [] probeLists = collection.probeSet().getAllProbeLists();
		
		JPanel topPanel = new JPanel();
		
		leftProbeListBox = new JComboBox(probeLists);
		leftProbeListBox.setPrototypeDisplayValue("No longer than this please");
		leftProbeListBox.addActionListener(new ActionListener() {
			
			public void actionPerformed(ActionEvent arg0) {
				vennDiagramPanel.setLeftProbeList((ProbeList)leftProbeListBox.getSelectedItem());
			}
		});
		
		
		rightProbeListBox = new JComboBox(probeLists);
		rightProbeListBox.setPrototypeDisplayValue("No longer than this please");
		rightProbeListBox.addActionListener(new ActionListener() {
			
			public void actionPerformed(ActionEvent arg0) {
				vennDiagramPanel.setRightProbeList((ProbeList)rightProbeListBox.getSelectedItem());
			}
		});
		
		topPanel.add(new JLabel("Left list"));
		topPanel.add(leftProbeListBox);
				
		topPanel.add(new JLabel("Right list"));
		topPanel.add(rightProbeListBox);
		
		JPanel buttonPanel = new JPanel();
		JButton closeButton = new JButton("Close");
		closeButton.addActionListener(new ActionListener() {
			
			public void actionPerformed(ActionEvent e) {
				setVisible(false);
				dispose();
			}
		});
		
		buttonPanel.add(closeButton);
		
		JButton saveButton = new JButton("Save");
		saveButton.addActionListener(new ActionListener() {
			
			public void actionPerformed(ActionEvent e) {
				ImageSaver.saveImage(vennDiagramPanel);
			}
		});
		
		buttonPanel.add(saveButton);
		
		getContentPane().add(buttonPanel,BorderLayout.SOUTH);
		
		
		getContentPane().add(topPanel,BorderLayout.NORTH);
		getContentPane().add(vennDiagramPanel,BorderLayout.CENTER);
		
		vennDiagramPanel.setLeftProbeList(probeLists[0]);
		vennDiagramPanel.setRightProbeList(probeLists[0]);
		
		setSize(600,400);
		setLocationRelativeTo(SeqMonkApplication.getInstance());
		setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
		setVisible(true);
		
		
	}
	
}
